import os
import sys

ROOT_FOLDER_NAME = ".lazysdk"
cache_root = "cache"
DEFAULT_PROJECT_ROOT_CACHE_PATH = os.path.join(sys.path[0], ROOT_FOLDER_NAME)  # 当前项目路径
DEFAULT_USER_HOME_CACHE_PATH = os.path.join(os.path.expanduser("~"), ROOT_FOLDER_NAME)  # 用户文件夹下的缓存
default_cache_directory = os.path.join(DEFAULT_USER_HOME_CACHE_PATH, cache_root)  # 默认用户缓存文件夹


def user_cache_path(folder_name: str = ".cache"):
    """
    返回用户缓存文件夹路径
    """
    return os.path.join(os.path.expanduser("~"), folder_name)



import sqlite3
# 连接到 SQLite 数据库
conn = sqlite3.connect('example.db')
# 创建一个游标对象
c = conn.cursor()
# 创建表
c.execute('''CREATE TABLE users (id INT, name TEXT)''')
# 插入数据
c.execute("INSERT INTO users VALUES (1, 'John Doe')")
# 提交事务
conn.commit()
# 查询数据
c.execute("SELECT * FROM users")
print(c.fetchall())
# 关闭连接
conn.close()


def insert(
        db: str,
        table: str,
        columns: List[str],
):
    pass

# class SQLite:
#     def __init__(self):
#         self.conn = sqlite3.connect('example.db')
#         self.c = conn.cursor()
#         self.c.execute('''CREATE TABLE users (id INT, name TEXT)''')
#         self.c.execute("INSERT INTO users VALUES (1, 'John Doe')")
#         self.conn.commit()
#         self.c.execute("SELECT * FROM users")
#
#     def INSERT_INTO():
#         pass


# from build.lib.lazysdk import lazychromedriver
from lazysdk import lazyfile
from lazysdk import lazyrequests
import subprocess
import platform
import zipfile
import tarfile
import sys
import re
import os
import shutil
from lazysdk import lazypath

# from webdriver_manager.core.utils import linux_browser_apps_to_cmd, windows_browser_apps_to_cmd, \
#     read_version_from_cmd

"""
借鉴webdriver_manager==4.0.1代码，但是原模块不好用，所以这里精简了
"""
ROOT_FOLDER_NAME = ".lazy_chromedriver"
DEFAULT_PROJECT_ROOT_CACHE_PATH = os.path.join(sys.path[0], ROOT_FOLDER_NAME)
DEFAULT_USER_HOME_CACHE_PATH = os.path.join(os.path.expanduser("~"), ROOT_FOLDER_NAME)
drivers_root = "drivers"
# drivers_directory = os.path.join(DEFAULT_USER_HOME_CACHE_PATH, drivers_root)
drivers_directory = os.path.join(drivers_root)


def determine_powershell():
    """Returns "True" if runs in Powershell and "False" if another console."""
    cmd = "(dir 2>&1 *`|echo CMD);&<# rem #>echo powershell"
    with subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.DEVNULL,
            stdin=subprocess.DEVNULL,
            shell=True,
    ) as stream:
        stdout = stream.communicate()[0].decode()
    return "" if stdout == "powershell" else "powershell"


def linux_browser_apps_to_cmd(*apps: str) -> str:
    """Create 'browser --version' command from browser app names.

    Result command example:
        chromium --version || chromium-browser --version
    """
    ignore_errors_cmd_part = " 2>/dev/null" if os.getenv(
        "WDM_LOG_LEVEL") == "0" else ""
    return " || ".join(f"{i} --version{ignore_errors_cmd_part}" for i in apps)


def windows_browser_apps_to_cmd(*apps: str) -> str:
    """Create analogue of browser --version command for windows."""
    powershell = determine_powershell()

    first_hit_template = """$tmp = {expression}; if ($tmp) {{echo $tmp; Exit;}};"""
    script = "$ErrorActionPreference='silentlycontinue'; " + " ".join(
        first_hit_template.format(expression=e) for e in apps
    )

    return f'{powershell} -NoProfile "{script}"'


def read_version_from_cmd(cmd, pattern):
    with subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stdin=subprocess.DEVNULL,
            shell=True,
    ) as stream:
        stdout = stream.communicate()[0].decode()
        version = re.search(pattern, stdout)
        version = version.group(0) if version else None
    return version


class ChromeType(object):
    GOOGLE = "google-chrome"
    CHROMIUM = "chromium"
    BRAVE = "brave-browser"
    MSEDGE = "edge"


class OSType(object):
    LINUX = "linux"
    MAC = "mac"
    WIN = "win"


PATTERN = {
    ChromeType.CHROMIUM: r"\d+\.\d+\.\d+",
    ChromeType.GOOGLE: r"\d+\.\d+\.\d+.\d+",  # 增加了最小版本
    ChromeType.MSEDGE: r"\d+\.\d+\.\d+",
    "brave-browser": r"\d+\.\d+\.\d+(\.\d+)?",
    "firefox": r"(\d+.\d+)",
}


def get_os_name():
    pl = sys.platform
    if pl == "linux" or pl == "linux2":
        return OSType.LINUX
    elif pl == "darwin":
        return OSType.MAC
    elif pl == "win32" or pl == "cygwin":
        return OSType.WIN


def get_os_type():
    pl = sys.platform
    if pl == "darwin":
        return f'mac-{platform.machine()}'
    elif pl == "win32" or pl == "cygwin":
        if platform.machine().endswith("64"):
            return 'win64'
        else:
            return 'win32'


class OperationSystemManager(object):

    def __init__(self, os_type=None):
        self._os_type = os_type

    @staticmethod
    def get_os_name():
        pl = sys.platform
        if pl == "linux" or pl == "linux2":
            return OSType.LINUX
        elif pl == "darwin":
            return OSType.MAC
        elif pl == "win32" or pl == "cygwin":
            return OSType.WIN

    @staticmethod
    def get_os_architecture():
        if platform.machine().endswith("64"):
            return 64
        else:
            return 32

    def get_os_type(self):
        if self._os_type:
            return self._os_type
        return f"{self.get_os_name()}{self.get_os_architecture()}"

    @staticmethod
    def is_arch(os_sys_type):
        if '_m1' in os_sys_type:
            return True
        return platform.processor() != 'i386'

    @staticmethod
    def is_mac_os(os_sys_type):
        return OSType.MAC in os_sys_type

    @staticmethod
    def get_browser_version_from_os(browser_type=None):
        """Return installed browser version."""
        cmd_mapping = {
            ChromeType.GOOGLE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "google-chrome",
                    "google-chrome-stable",
                    "google-chrome-beta",
                    "google-chrome-dev",
                ),
                OSType.MAC: r"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Google\Chrome\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Google Chrome").version',
                ),
            },
            ChromeType.CHROMIUM: {
                OSType.LINUX: linux_browser_apps_to_cmd("chromium", "chromium-browser"),
                OSType.MAC: r"/Applications/Chromium.app/Contents/MacOS/Chromium --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Chromium\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Chromium").version',
                ),
            },
            ChromeType.BRAVE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "brave-browser", "brave-browser-beta", "brave-browser-nightly"
                ),
                OSType.MAC: r"/Applications/Brave\ Browser.app/Contents/MacOS/Brave\ Browser --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\BraveSoftware\Brave-Browser\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\BraveSoftware Brave-Browser").version',
                ),
            },
            ChromeType.MSEDGE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "microsoft-edge",
                    "microsoft-edge-stable",
                    "microsoft-edge-beta",
                    "microsoft-edge-dev",
                ),
                OSType.MAC: r"/Applications/Microsoft\ Edge.app/Contents/MacOS/Microsoft\ Edge --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    # stable edge
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Microsoft\EdgeUpdate\Clients\{56EB18F8-8008-4CBD-B6D2-8C97FE7E9062}").pv',
                    # beta edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge Beta\BLBeacon").version',
                    # dev edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge Dev\BLBeacon").version',
                    # canary edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge SxS\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge SxS\BLBeacon").version',
                    # highest edge
                    r"(Get-Item (Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\msedge.exe').'(Default)').VersionInfo.ProductVersion",
                    r"[System.Diagnostics.FileVersionInfo]::GetVersionInfo((Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\msedge.exe').'(Default)').ProductVersion",
                    r"Get-AppxPackage -Name *MicrosoftEdge.* | Foreach Version",
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Microsoft Edge").version',
                ),
            },
            "firefox": {
                OSType.LINUX: linux_browser_apps_to_cmd("firefox"),
                OSType.MAC: r"/Applications/Firefox.app/Contents/MacOS/firefox --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Mozilla Firefox\firefox.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Mozilla Firefox\firefox.exe").VersionInfo.FileVersion',
                    r"(Get-Item (Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\firefox.exe').'(Default)').VersionInfo.ProductVersion",
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Mozilla\Mozilla Firefox").CurrentVersion',
                ),
            },
        }

        try:
            cmd_mapping = cmd_mapping[browser_type][OperationSystemManager.get_os_name()]
            pattern = PATTERN[browser_type]
            version = read_version_from_cmd(cmd_mapping, pattern)
            return version
        except Exception:
            return None
            # raise Exception("Can not get browser version from OS")


def get_browser_version(browser_type: str = "google-chrome"):
    """
    获取浏览器的版本，例如：144.0.7559.60
    """
    return OperationSystemManager().get_browser_version_from_os(browser_type=browser_type)


def find_driver_url(
        browser_version: str = None
):
    """
    查找匹配同一个大版本下的最新子版本的chromedriver
    """
    if not browser_version:
        browser_version = get_browser_version()
    os_type = get_os_type()
    url = "https://repo.huaweicloud.com/chromedriver/"
    response = lazyrequests.lazy_requests(
        method="GET",
        url=f"{url}.index.json",
    )
    drivers = response["chromedriver"]
    driver_versions = list(drivers.keys())
    browser_version_p = ".".join(browser_version.split(".")[:-1])
    match_versions = list()
    for each_version in driver_versions:
        each_version_p = ".".join(each_version.split(".")[:-1])
        if browser_version_p == each_version_p:
            match_versions.append(each_version)
    match_version_max = max(match_versions)
    match_version_max_files = drivers[match_version_max]["files"]
    for each_file in match_version_max_files:
        if each_file.split(".")[-2].endswith(os_type):
            match_chromedriver_url = f"{url}{each_file}"
            return match_chromedriver_url
        else:
            continue
    return None


# def get_driver_download_url(
#         driver_url='https://edgedl.me.gvt1.com/edgedl/chrome/chrome-for-testing',
# ):
#     os_type = get_os_type()
#     driver_version = OperationSystemManager().get_browser_version_from_os(browser_type='google-chrome')
#     print("driver_version:", driver_version)
#     return f"{driver_url}/{driver_version}/{os_type}/chromedriver-{os_type}.zip"


# def save_archive_file(file: File, directory: str):
#     os.makedirs(directory, exist_ok=True)
#
#     archive_path = f"{directory}{os.sep}{file.filename}"
#     with open(archive_path, "wb") as code:
#         code.write(file.content)
#     if not os.path.exists(archive_path):
#         raise FileExistsError(f"No file has been saved on such path {archive_path}")
#     return Archive(archive_path)
#
#
def download_driver(overwrite: bool = False):
    """
    根据当前系统的chrome版本下载对应版本的driver
    """
    from lazysdk import lazyfile
    browser_version = get_browser_version()  # 获取浏览器版本

    drivers_directory_version = os.path.join(drivers_directory, browser_version)
    print("drivers_directory_version:", drivers_directory_version)
    driver_dir = os.path.join(drivers_directory_version, "chromedriver")
    if os.path.exists(drivers_directory_version):
        # 路径已经存在
        return driver_dir
    else:
        # 路径不存在，需要下载
        os.makedirs(drivers_directory_version, exist_ok=True)  # 创建driver下载目录

        driver_download_url = find_driver_url(browser_version=browser_version)
        # print("driver_download_url:", driver_download_url)
        filename = extract_filename_from_url(driver_download_url)
        # print('filename:', filename)
        archive_path = os.path.join(drivers_directory_version, filename)
        # archive_path = f"{directory}{os.sep}{file.filename}"
        # print("archive_path:", archive_path)
        file = lazyfile.download(
            url=driver_download_url,
            path=drivers_directory_version
        )['file_dir']
        # print("file:", file)
        unpack_archive_files = unpack_archive(
            archive_file=file,
        )
        for each_file in unpack_archive_files:
            if each_file.endswith("/chromedriver"):
                shutil.copy2(each_file, driver_dir)
                os.chmod(path=driver_dir, mode=755)
                return driver_dir
        return None


class LinuxZipFileWithPermissions(zipfile.ZipFile):
    """Class for extract files in linux with right permissions"""

    def extract(self, member, path=None, pwd=None):
        if not isinstance(member, zipfile.ZipInfo):
            member = self.getinfo(member)

        if path is None:
            path = os.getcwd()

        ret_val = self._extract_member(member, path, pwd)  # noqa
        attr = member.external_attr >> 16
        os.chmod(ret_val, attr)
        return ret_val


def unpack_archive(
        archive_file,
        target_dir: str = None
):
    if archive_file.endswith(".zip"):
        return lazyfile.unzip(file=archive_file)
    elif archive_file.endswith(".tar.gz"):
        return __extract_tar_file(archive_file, target_dir)


def __extract_zip(archive_file, to_directory):
    zip_class = zipfile.ZipFile
    archive = zip_class(archive_file)
    try:
        archive.extractall(to_directory)
    except Exception as e:
        if e.args[0] not in [26, 13] and e.args[1] not in [
            "Text file busy",
            "Permission denied",
        ]:
            raise e
        file_names = []
        for n in archive.namelist():
            if "/" not in n:
                file_names.append(n)
            else:
                file_path, file_name = n.split("/")
                full_file_path = os.path.join(to_directory, file_path)
                source = os.path.join(full_file_path, file_name)
                destination = os.path.join(to_directory, file_name)
                os.replace(source, destination)
                file_names.append(file_name)
        return sorted(file_names, key=lambda x: x.lower())
    return archive.namelist()


def __extract_tar_file(archive_file, to_directory):
    try:
        tar = tarfile.open(archive_file.file_path, mode="r:gz")
    except tarfile.ReadError:
        tar = tarfile.open(archive_file.file_path, mode="r:bz2")
    members = tar.getmembers()
    tar.extractall(to_directory)
    tar.close()
    return [x.name for x in members]


def extract_filename_from_url(url):
    """
    从url中获取文件名
    """
    # Split the URL by '/'
    url_parts = url.split('/')
    # Get the last part of the URL, which should be the filename
    filename = url_parts[-1]
    # Decode the URL-encoded filename
    filename = os.path.basename(filename)
    return filename



import platform
import datetime
import socket
import os
import hashlib


def get_mac_address() -> str:
    """
    获取mac地址，只有联网才能生效
    """
    import netifaces
    try:
        default_gate_way, default_nic_name = netifaces.gateways()['default'][netifaces.AF_INET]  # 获取默认网关和网卡名称
        default_nic_mac_addr = netifaces.ifaddresses(default_nic_name)[netifaces.AF_LINK][0]['addr']  # 默认网卡的mac地址
        return default_nic_mac_addr
    except:
        return ''


def get_net_info():
    import netifaces
    default_gate_way, default_nic_name = netifaces.gateways()['default'][netifaces.AF_INET]  # 获取默认网关和网卡名称
    default_nic_mac_addr = netifaces.ifaddresses(default_nic_name)[netifaces.AF_LINK][0]['addr']  # 默认网卡的mac地址
    default_ip_addr = netifaces.ifaddresses(default_nic_name)[netifaces.AF_INET][0]['addr']  # 本地ip
    default_ip_netmask = netifaces.ifaddresses(default_nic_name)[netifaces.AF_INET][0]['netmask']  # 子网掩码
    default_ip_broadcast = netifaces.ifaddresses(default_nic_name)[netifaces.AF_INET][0]['netmask']
    return {'mac': default_nic_mac_addr, 'local_ip': default_ip_addr, 'netmask': default_ip_netmask, 'broadcast': default_ip_broadcast}


def platform_info() -> dict:
    """
    获取当前环境的信息
    """
    platform_processor = platform.processor()
    platform_architecture = platform.architecture()

    hostname = socket.gethostname()

    res = dict()

    res["node"] = platform.node()  # 计算机的网络名称/主机名

    res["local_datetime"] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

    res["platform"] = platform.platform()  # 操作系统名称及版本号
    res["system"] = platform.system()  # 操作系统名称
    res["version"] = platform.version()  # 操作系统详细版本
    res["release"] = platform.release()  # 操作系统大版本

    res["platform_machine"] = platform.machine()  # 计算机类型/平台架构
    res["platform_processor"] = platform_processor  # 计算机处理器信息/处理器名称
    res["platform_architecture"] = platform_architecture  # 操作系统的位数

    res["hostname"] = hostname
    res["mac"] = get_mac_address()

    res["cpu_count"] = os.cpu_count()

    import locale
    res["encoding"] = locale.getpreferredencoding()  # 获取系统编码类型

    return res


def equipment_calibration(detail=False):
    import wmi
    c = wmi.WMI()

    # 硬盘序列号
    equipment_information_pre = ''
    equipment_information_list = []
    for physical_disk in c.Win32_DiskDrive():
        equipment_information_pre = equipment_information_pre + physical_disk.SerialNumber
        equipment_information_list.append(physical_disk.SerialNumber)

    # CPU序列号
    cpu_information_list = []
    for cpu in c.Win32_Processor():
        equipment_information_pre = equipment_information_pre + cpu.ProcessorId.strip()
        cpu_information_list.append(cpu.ProcessorId.strip())

    # 主板序列号
    main_board_information_list = []
    for board_id in c.Win32_BaseBoard():
        equipment_information_pre = equipment_information_pre + board_id.SerialNumber
        main_board_information_list.append(board_id.SerialNumber)

    equipment_information = hashlib.md5(equipment_information_pre.encode(encoding='UTF-8')).hexdigest()

    if detail:
        return {
            'hard_disk': equipment_information_list,
            'cpu': cpu_information_list,
            'main_board': main_board_information_list,
            'equipment_code': equipment_information
        }
    else:
        return equipment_information


#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import subprocess
import platform
import shutil
import sys
import os
import filetype
current_user_path = os.path.abspath('.')
if platform.system() == 'Windows':
    path_separator = '\\'
else:
    path_separator = '/'


def make_path(
        path: str = None,
        overwrite: bool = False
) -> bool:
    """
    创建目录
    """
    if path is None:
        return False
    else:
        if overwrite:
            delete(path_or_file=path)
        os.makedirs(name=path, exist_ok=True)
        return True


def make_data_path(
        file_name: str
):
    file_name = os.path.basename(file_name).replace(".py", "")
    path = os.path.join('data', file_name)
    if os.path.exists(path) is True:
        pass
    else:
        make_path("data")
        os.mkdir(path)
    return path


def project_path(
        project_name
):
    # 获取项目的根目录，需要输入项目名称
    cur_path = os.path.abspath(os.path.dirname(__file__))
    _project_path = cur_path[:cur_path.find("%s%s" % (project_name, path_separator)) + len("%s%s" % (project_name, path_separator))]
    return _project_path


def file_path(__file__):
    """
    os.path.dirname(os.path.abspath(__file__))  单纯的文件地址
    os.path.dirname(os.path.realpath(__file__))  可能会存在的文件指向的真实地址
    """
    return os.path.dirname(os.path.abspath(__file__))


def get_all_top_dir():
    """
    枚举当前文件所处的文件夹的所有明细路径
    """
    file_dir = sys.argv[0]
    dir_list = file_dir.split(path_separator)
    dir_list_length = len(dir_list)
    new_dir_list = list()
    for i in range(dir_list_length):
        dir_list.pop(dir_list_length - i - 1)
        if len(dir_list) > 0:
            temp_dir = path_separator.join(dir_list)
            if len(temp_dir) > 0:
                new_dir_list.append(temp_dir)
            else:
                continue
        else:
            continue
    return new_dir_list


def visit_dir(
        path
):
    total_size = 0
    file_num = 0
    dir_num = 0
    for lists in os.listdir(path):
        sub_path = os.path.join(path, lists)
        # print(sub_path)
        if os.path.isfile(sub_path):
            file_num = file_num+1
            # print(fileNum)# 统计文件数量
            total_size = total_size+os.path.getsize(sub_path)  # 文件总大小
        elif os.path.isdir(sub_path):
            dir_num = dir_num+1  # 统计文件夹数量
            visit_dir(sub_path)  # 递归遍历子文件夹
    return file_num


def file_list(
        file_dir=os.path.dirname(os.path.realpath(__file__)),
        with_path=False
):
    """
    对某个路径枚举路径下的文件列表
    """
    dir_file_list = os.listdir(file_dir)
    if with_path:
        dir_file_list_f = list()
        for each in dir_file_list:
            dir_file_list_f.append(os.path.join(file_dir, each))
        return dir_file_list_f
    else:
        return dir_file_list


def open_path(path):
    """
    打开某个路径/文件
    """
    if platform.system() == 'Windows':
        os.startfile(path)  # Windows上打开文件
    else:
        subprocess.check_call(['open', path])  # 非Windows上打开文件


def path_clean(content):
    """
    清除路径前后可能出现的引号
    """
    if content[0] == '"' and content[-1] == '"':
        content = content[1:-1]
    elif content[0] == '“' and content[-1] == '”':
        content = content[1:-1]
    elif content[0] == "'" and content[-1] == "'":
        content = content[1:-1]
    else:
        pass
    return content


def file_info(
        file
):
    """
    获取文件信息
    """
    res = dict()
    res['file'] = file
    res['path_exists'] = os.path.exists(file)  # 文件/路径是否存在
    if os.path.exists(file):
        # 文件存在，继续获取详细信息
        create_time = os.path.getctime(file)
        if os.path.isdir(file):
            res['path_type'] = 'dir'
            res['path_name'] = os.path.basename(file)
        elif os.path.isfile(file):
            res['path_type'] = 'file'
            res['file_name'] = os.path.basename(file)
            guess_kind = filetype.guess(file)  # 猜测文件类型
            if guess_kind:
                res['file_extension'] = guess_kind.extension  # 后缀名，例如：mp4
                res['file_type'] = guess_kind.mime  # 文件类型，例如：video/mp4
        else:
            res['path_type'] = ''

        res['path_isabs'] = os.path.isabs(file)  # 是否为绝对路径
        res['size'] = os.path.getsize(file)  # 获取大小
        res['dir_name'] = os.path.dirname(file)  # 上层目录名
    else:
        # 文件不存在
        pass
    return res


def delete(path_or_file):
    """
    删除目录/文件
    """
    if os.path.exists(path_or_file):
        if os.path.isdir(path_or_file):
            shutil.rmtree(path_or_file)
        else:
            os.remove(path_or_file)
    else:
        return


def get_folder_name(dir_or_path: str):
    """
    获取文件夹名称
    """
    if os.path.isdir(dir_or_path):
        # 这里多做一步处理，避免有可能路径后面再有个路径分割符号不能识别的情况
        return os.path.basename(os.path.abspath(dir_or_path))
    elif os.path.isfile(dir_or_path):
        tmp_dir = os.path.dirname(dir_or_path)
        return os.path.basename(tmp_dir)
    else:
        return


def path_rename(
        folder_path,
        new_folder_name
):
    """
    文件夹重命名
    """
    if os.path.isdir(folder_path):
        second_last_path, last_path = os.path.split(os.path.abspath(folder_path))
        new_path = os.path.join(second_last_path, new_folder_name)
        os.rename(folder_path, new_path)
        return new_path
    else:
        return


def exe_path():
    """
    获取当前脚本的绝对路径
    """
    return os.path.dirname(os.path.realpath(sys.argv[0]))


def home():
    from pathlib import Path
    # 获取用户主目录
    return Path.home()  # 输出示例: C:\Users\Username 或 /home/username



from tinydb import TinyDB, Query


def insert(
        save_data: dict,
        db_name: str = "db.json",
):
    db = TinyDB(db_name)
    return db.insert(save_data)


def insert_multiple(
        save_data: list,
        db_name: str = "db.json",
):
    db = TinyDB(db_name)
    return db.insert_multiple(save_data)


def get(
        db_name: str = "db.json",
        _all: bool = False,
        search_key: str = None,
        search_value = None
):
    db = TinyDB(db_name)
    tb = Query()
    if _all:
        return db.all()  # 输出所有
    else:
        if search_key is not None and search_value is not None:
            return db.search(eval(f"tb.{search_key}") == search_value)
        else:
            return db.all()


def get_all(db_name: str):
    db = TinyDB(db_name)
    return db.all()


def upsert(
        data: list,
        db_name: str = "db.json",
        key: str = None,
):
    db = TinyDB(db_name)
    tb = Query()
    for each_data in data:
        key_value = each_data.get(key)
        db.upsert(each_data, eval(f"tb.{key}") == key_value)


if __name__ == '__main__':
    # print(insert_multiple([{"a": 2}, {"a": 2}]))
    # print(get({"name": "John"}))

    # print(upsert(
    #     data=[
    #         {"a": 2, "b": 333},
    #         {"a": 3, "b": 4, "c": 5, "d": 6, "e": 7},
    #     ],
    #     key="a"
    # ))

    print(get(
        search_key="a",
        search_value=2,
    ))


#!/usr/bin/env python3
# coding = utf8
"""
    这个模块的作用非常简单
    就是将数据表显示的好看一点
"""

import json
import datetime
from rich.console import Console
console = Console()


def show_df(df):
    """
    更好的方式：
    利用prettytable对输出结果进行美化 https://www.cnblogs.com/shuchang/articles/11029456.html
    :param df:
    :return:
    """
    from prettytable import PrettyTable  # prettytable==3.8.0
    x = PrettyTable()
    # x.padding_width = 1  # One space between column edges and contents (default)
    x.add_column(
        fieldname='data_index',
        column=df.index
    )
    for col in df.columns.values:  # df.columns.values的意思是获取列的名称
        x.add_column(col, df[col])
    print(x)


class DateEncoder(json.JSONEncoder):
    def default(self, obj):
        """
        只要检查到了是bytes类型的数据就把它转为str类型
        :param obj:
        :return:
        """
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, bytes):
            return str(obj, encoding='utf-8')
        else:
            return json.JSONEncoder.default(self, obj)


def show_dict(
        obj: dict,
        use_rich: bool = False
):
    """
    显示字典dict
    """
    if use_rich:
        console.print(obj)
    else:
        show_res = json.dumps(
            obj=obj,
            sort_keys=True,
            indent=4,
            cls=DateEncoder,
            ensure_ascii=False
        )
        print(show_res)




from . import lazytime
import re


def clean_id_card(
        id_card
):
    """
    清洗身份证号码，只保留数字和X，不会改变字符串长度
    """
    # 定义正则表达式，匹配数字和字母X
    pattern = re.compile(r'[0-9Xx]+')

    # 使用正则表达式匹配身份证号码中的数字和字母X
    matches = pattern.findall(id_card)

    # 将匹配结果拼接成身份证号码
    cleaned_id_card = ''.join(matches)

    return cleaned_id_card.upper()


def validate_id_card(
        id_card
):
    """
    校验身份证号码，正确输出True，错误输出False
    """
    # 去除干扰字符，只保留身份证号码中的数字和字母X
    id_card = clean_id_card(id_card=id_card)

    # 身份证号码必须为18位
    if len(id_card) != 18:
        return False

    # 校验前17位是否都是数字
    if not id_card[:-1].isdigit():
        return False

    # 校验最后一位是否是数字或者字母X
    if not (id_card[-1].isdigit() or id_card[-1] == 'X'):
        return False

    # 加权因子
    weight_factors = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
    # 校验码对应值
    validate_values = '10X98765432'
    # 计算校验位
    sum_value = sum(int(id_card[i]) * weight_factors[i] for i in range(17))
    validate_index = sum_value % 11
    validate_code = validate_values[validate_index]

    # 校验最后一位
    return id_card[-1] == validate_code


def id_card_info(
        id_card: str
):
    """
    在身份证号码中，第17位是性别位，用于表示持证人的性别。性别位的奇偶性规则如下：

    奇数表示男性（例如，1、3、5、7、9）
    偶数表示女性（例如，0、2、4、6、8）

    通过身份证号码的性别位，我们可以快速判断持证人的性别。
    """
    id_card = clean_id_card(id_card=id_card)
    if validate_id_card(id_card=id_card):
        ana_id_info = dict()
        ana_id_info['id_str'] = id_card

        year_now = lazytime.get_year()
        ana_id_info['year_now'] = year_now

        year = id_card[6:10]
        month = id_card[10:12]
        day = id_card[12:14]
        ana_id_info['year'] = year
        ana_id_info['month'] = month
        ana_id_info['day'] = day

        age = year_now - eval(year)
        ana_id_info['age'] = age

        gender_num = id_card[16]
        s = eval(gender_num)
        if s % 2 == 0:
            gender_str = "女"
        else:
            gender_str = "男"
        ana_id_info['gender_num'] = gender_num
        ana_id_info['gender_str'] = gender_str
        ana_id_info['msg'] = 'success'
        ana_id_info['code'] = 0
        return ana_id_info
    else:
        return {
            'code': -1,
            'msg': '身份证号码错误'
        }



import os
import platform
from lazysdk import lazychromedriver

# browser_version = lazychromedriver.get_browser_version()
# print(browser_version)
# driver_url = lazychromedriver.find_driver_url()
# print(driver_url)
print(lazychromedriver.download_driver())

"""
chromedriver镜像地址：
https://repo.huaweicloud.com/chromedriver/
https://repo.huaweicloud.com/chromedriver/.index.json
"""


#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import base64


def lazy_b64decode(s):
    """
    base64加密后的字符串位数一定是4的整数倍，这里在解码时判断，缺多少补多少
    :param s: 需要解密的字串
    """
    add_count = len(s) % 4
    if add_count > 0:
        for _ in range(4-add_count):
            s += '='
    s_decode = base64.b64decode(s=s).decode()
    return s_decode


def lazy_b64encode(s):
    """
    base64编码
    """
    return base64.b64encode(s.encode('utf-8')).decode()





def split(
        list_in: list,
        split_gap: int
):
    """
    将输入的list按照split_gap进行分割
    """
    list_out = list()
    split_num = 0
    for i in range(len(list_in)):
        if i % split_gap == 0:
            list_out.append(list())
            split_num += 1
        list_out[split_num-1].append(list_in[i])
    return list_out


def list_difference(
        list_left: list,
        list_right: list,
        keep_sort: bool = False
):
    """
    返回两个list的差集，效果等于list_left-list_right，返回无序；

    在 Python 中，可以使用集合（`set`）的特性来取两个列表的差集。差集是指只在一个集合中出现而在另一个集合中没有的元素。
    使用集合的 `difference()` 方法可以实现列表的差集操作。
    需要注意的是，集合是无序的，因此在结果中元素的顺序可能与原始列表不同。如果需要保留原始列表的顺序，可以使用列表推导式来实现差集操作
    """
    if keep_sort:
        return [x for x in list_left if x not in list_right]  # 返回有序
    else:
        return list(set(list_left).difference(list_right))  # 返回无序


if __name__ == '__main__':
    print(list_difference(
        list_left=[4,1,2],
        list_right=[2,3],
        keep_sort=True
    ))


#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="lazysdk",
    version="0.2.7",
    description="基于Python的懒人包",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="ZeroSeeker",
    author_email="zeroseeker@foxmail.com",
    url="https://gitee.com/ZeroSeeker/lazysdk",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'showlog>=0.0.6',
        'requests>=2.31.0',
        'envx>=1.0.1',
        'pytz==2022.1',
        'ua_parser==0.10.0',
        'openpyxl==3.0.9',
        'xlrd==2.0.1',
        'm3u8==3.5.0',
        'pycryptodome==3.10.1',
        'filetype==1.2.0',
        'netifaces==0.11.0',
        'user_agents==2.2.0',
        'rich>=13.5.2',
        'urllib3==1.26.10'  # 之前为urllib3==1.23
    ]
)
